# frozen_string_literal: true

module  Apiphobic
module  Parameters
class   Filter
  FLOAT           = /\A[\-+]?[_\d]+\.[_\d]+\z/
  INTEGER         = /\A[\-+]?[_\d]+\z/
  ISO8601         = /\d{4}\-\d{2}\-\d{2}T\d{2}\:\d{2}\:\d{2}(?:[+\-]\d{2}\:\d{2}|Z)/
  NUMERICAL       = /[\-+]?[_\d]+(?:\.\d+)?/

  DATE_RANGE      = /\A(#{ISO8601})\.\.\.?(#{ISO8601})\z/
  NUMERICAL_RANGE = /\A(#{NUMERICAL})\.\.\.?(#{NUMERICAL}|Infinity)\z/

  attr_accessor :raw_parameters

  def initialize(raw_parameters)
    self.raw_parameters = raw_parameters || {}
  end

  def present?
    compacted_parameters.any?
  end

  def each_with_object(memoized)
    compacted_parameters.each do |name, value|
      memoized = yield [name, format_value(value)], memoized
    end

    memoized
  end

  private

  def compacted_parameters
    @compacted_parameters ||= raw_parameters.reject do |name, value|
      name == 'query' ||
      value == '' ||
      value.nil?
    end
  end

  # rubocop:disable Lint/AssignmentInCondition, Metrics/AbcSize,
  # rubocop:disable Metrics/CyclomaticComplexity, Metrics/PerceivedComplexity
  def format_value(value)
    return value unless value.is_a?(::String)

    if range_points  = value.match(DATE_RANGE)
      exclusive      = value.include?('...')
      starting_point = ::Time.iso8601(range_points[1])
      ending_point   = ::Time.iso8601(range_points[2])

      ::Range.new(starting_point, ending_point, exclusive)
    elsif range_points = value.match(NUMERICAL_RANGE)
      exclusive        = value.include?('...')
      starting_point   = string_to_number(range_points[1])
      ending_point     = string_to_number(range_points[2])

      ::Range.new(starting_point, ending_point, exclusive)
    elsif value.match?(FLOAT)
      Float(value.delete('_'))
    elsif value.match?(INTEGER)
      Integer(value.delete('_'))
    elsif value.match?(ISO8601)
      ::Time.iso8601(value)
    else
      value
    end
  end
  # rubocop:enable Metrics/CyclomaticComplexity, Metrics/PerceivedComplexity
  # rubocop:enable Lint/AssignmentInCondition, Metrics/AbcSize,

  def string_to_number(other)
    other = other.delete('_')

    if other == 'Infinity'
      9_999_999
    elsif other.match?(FLOAT)
      other.to_f
    elsif other.match?(INTEGER)
      other.to_i
    end
  end
end
end
end
